#
# Borgend borg launcher / processor
#

import json
import logging
import os
from subprocess import Popen, PIPE
from config import settings, arglistify

necessary_opts=['--log-json', '--progress']

necessary_opts_for={
    'create': ['--json'],
    'info': ['--json'],
    'list': ['--json'],
}

class BorgInstance:
    def __init__(self, operation, archive_or_repository, args, argsl):
        self.operation=operation;
        self.args=args;
        self.archive_or_repository=archive_or_repository;
        self.argsl=argsl;

    def construct_cmdline(self):
        cmd=([settings['borg']['executable']]+necessary_opts+
             arglistify(settings['borg']['common_parameters'])+
             [self.operation])
        tmp1=self.operation+'_parameters'
        if tmp1 in settings['borg']:
            cmd=cmd+arglistify(settings['borg'][tmp1])

        if self.operation in necessary_opts_for:
            cmd=cmd+necessary_opts_for[self.operation]

        return cmd+arglistify(self.args)+[self.archive_or_repository]+self.argsl

    def launch(self, passphrase=None):
        cmd=self.construct_cmdline()

        logging.info('Launching ' + str(cmd))

        env=None
        if passphrase:
            env=os.environ.copy()
            env['BORG_PASSPHRASE']=passphrase

        # Workaround: if launched is a standalone app created with py2app,
        # borg will fail unless Python environment is reset.
        # TODO: Of course, this will fail if the system needs the variables
        # PYTHONPATH or PYTHONHOME set to certain values.
        if '_PY2APP_LAUNCHED_' in env and env['_PY2APP_LAUNCHED_']=='1':
            if env==None:
                env=os.environ.copy()
            del env['PYTHONPATH']
            del env['PYTHONHOME']

        self.proc=Popen(cmd, env=env, stdout=PIPE, stderr=PIPE, stdin=PIPE)

        # We don't do passphrase input etc.
        self.proc.stdin.close()

    def read_result(self):
        stream=self.proc.stdout
        line=stream.read(-1)
        if line==b'':
            logging.debug('Borg stdout pipe EOF?')
            return None

        try:
            return json.loads(line)
        except:
            logging.warning('JSON parse failed on: "%s"' % line)
            return None

    def read_log(self):
        stream=self.proc.stderr
        try:
            line=stream.readline()
        except err:
            logging.debug('Pipe read failed: %s' % str(err))

            return {'type': 'log_message',
                    'levelname': 'CRITICAL',
                    'name': 'borgend.instance.BorgInstance',
                    'msgid': 'Borgend.Exception',
                    'message': err}

        if line==b'':

            logging.debug('Borg stderr pipe EOF?')

            return None

        try:
            res=json.loads(line)
            if 'type' not in res:
                res['type']='UNKNOWN'
            return res
        except:
            logging.debug('JSON parse failed on: "%s"' % str(line))

            errmsg=line
            for line in iter(stream.readline, b''):
                errmsg=errmsg+line

            return {'type': 'log_message',
                    'levelname': 'ERROR',
                    'name': 'borgend.instance.BorgInstance',
                    'msgid': 'Borgend.JSONFail',
                    'message': str(errmsg)}

    def terminate(self):
        self.proc.terminate()

    def wait(self):
        return self.proc.wait() is not None

    def has_terminated(self):
        return self.proc.poll() is not None

